        
//*****************************************************************************************
//                                                                            	          *
//      EtherCAT driver for LAN9252                                          	          *
//                                                                            	          *  
//***************************************************************************************** 
 
#include <p18f26k22.h>
#include <delays.h> 
             
#include "main.h"  
#include "easycat.h"     



//-- external variables ------------------------------------------------------------------

extern BUFFOUT Out;
extern BUFFIN In;


//---- local functions ----------------------------------------------------------------------------

    void          SPIWriteRegisterDirect   (unsigned short Address, unsigned long DataOut);
    unsigned long SPIReadRegisterDirect    (unsigned short Address, unsigned char Len);
    
    void          SPIWriteRegisterIndirect (unsigned long  DataOut, unsigned short Address);
    unsigned long SPIReadRegisterIndirect  (unsigned short Address, unsigned char Len); 
    
    void          SPIReadProcRamFifo(void);    
    void          SPIWriteProcRamFifo(void);  

    unsigned char SPI1Transfer (unsigned char Value); 

 
//---- EasyCAT board initialization ---------------------------------------------------------------

unsigned char EasyCAT_Init()
{
  unsigned char i;
  ULONG TempLong;

  Delay10KTCYx(200);                                                // wait 125mS

  for (i=0; i<10; i++)                                              // check test register
  {                                                                 //
    Delay10KTCYx(20);                                               // wait 12.5mS   
    TempLong.Long = SPIReadRegisterDirect (BYTE_TEST, 4);           //
    if (TempLong.Long == 0x87654321)                                //
      break;                                                        //
  }                                                                 //
  if (i == 10)                                                      //
    return false;                                                   // we waited for too long: error

  for (i=0; i<10; i++)                                              // check ready flag
  {                                                                 //
    Delay10KTCYx(20);                                               // wait 12.5 mS
    TempLong.Long = SPIReadRegisterDirect (HW_CFG, 4);              //
    if ((TempLong.Long & READY) != 0)                               //
      break;                                                        //
  }                                                                 //        
  if (i == 10)                                                      //
    return false;                                                   // we waited for too long: error

  return true;                                                      // EasyCAT initialized
}


//---- EtherCAT task ------------------------------------------------------------------------------

void EasyCAT_MainTask()                           // must be called cyclically by the application

{
  unsigned char WatchDog = 0;
  unsigned char Operational = 0; 
  unsigned char i;
  ULONG TempLong; 
 
  
  TempLong.Long = SPIReadRegisterIndirect (WDOG_STATUS, 1); // read watchdog status
  if ((TempLong.Byte[0] & 0x01) == 0x01)                    //
    WatchDog = 0;                                           // set/reset the corrisponding flag
  else                                                      //
    WatchDog = 1;                                           //
    
  TempLong.Long = SPIReadRegisterIndirect (AL_STATUS, 1);   // read the EtherCAT State Machine status
  if ((TempLong.Byte[0] & 0x0F) == ESM_OP)                  // to see if we are in operational state
    Operational = 1;                                        //
  else                                                      // set/reset the corrisponding flag
    Operational = 0;                                        //

                                                            //--- process data transfert ----------
                                                            //                                                        
  if (WatchDog | !Operational)                              // if watchdog is active or we are 
  {                                                         // not in operational state, reset 
    for (i=0; i<4; i++)                                     // the output buffer
    Out.Long[i] = 0;                                        //
    Led = 0;                                                // led off
                                                            // (and disables outputs)    
  }
  
  else                                                      
  {                                                         
    SPIReadProcRamFifo();                                   // otherwise transfer process data from 
                                                            // the EtherCAT core to the output buffer  
    Led = 1;                                                // led on
                                                            // (and enables outputs)
  }

  SPIWriteProcRamFifo();                                    // we always transfer process data from
}                                                           // the input buffer to the EtherCAT core  
  

    
//---- read a directly addressable registers  -----------------------------------------------------

unsigned long SPIReadRegisterDirect (unsigned short Address, unsigned char Len)

                                                   // Address = register to read
                                                   // Len = number of bytes to read (1,2,3,4)
                                                   //
                                                   // a long is returned but only the requested bytes
                                                   // are meaningful, starting from LsByte                                                 
{

  unsigned char i;
  ULONG Result; 
  UWORD Addr;
  Addr.Word = Address;

  
  Spi1Cs_ = 0;	                                            // SPI chip select enable

  SPI1Transfer(COMM_SPI_READ);                              // SPI read command
  SPI1Transfer(Addr.Byte[1]);                               // address of the register
  SPI1Transfer(Addr.Byte[0]);                               // to read, MsByte first
 
  for (i=0; i<Len; i++)                                     // read the requested number of bytes
  {                                                         // LsByte first 
    Result.Byte[i] = SPI1Transfer(DUMMY_BYTE);              //
  }                                                         //    
  
  Spi1Cs_ = 1;                                              // SPI chip select disable 
 
  return Result.Long;                                       // return the result
}


//---- write a directly addressable registers  ----------------------------------------------------

void SPIWriteRegisterDirect (unsigned short Address, unsigned long DataOut)

                                                   // Address = register to write
                                                   // DataOut = data to write
{ 
  ULONG Data; 
  UWORD Addr;
  Addr.Word = Address;
  Data.Long = DataOut;    

  Spi1Cs_ = 0;                                              // SPI chip select enable  
  
  SPI1Transfer(COMM_SPI_WRITE);                             // SPI write command
  SPI1Transfer(Addr.Byte[1]);                               // address of the register
  SPI1Transfer(Addr.Byte[0]);                               // to write MsByte first

  SPI1Transfer(Data.Byte[0]);                               // data to write 
  SPI1Transfer(Data.Byte[1]);                               // LsByte first
  SPI1Transfer(Data.Byte[2]);                               //
  SPI1Transfer(Data.Byte[3]);                               //
 
  Spi1Cs_ = 1;                                              // SPI chip select disable   
}


//---- read an undirectly addressable registers  --------------------------------------------------

unsigned long SPIReadRegisterIndirect (unsigned short Address, unsigned char Len)

                                                   // Address = register to read
                                                   // Len = number of bytes to read (1,2,3,4)
                                                   //
                                                   // a long is returned but only the requested bytes
                                                   // are meaningful, starting from LsByte                                                  
{
  ULONG TempLong;
  UWORD Addr;
  Addr.Word = Address;
                                                            // compose the command
                                                            //
  TempLong.Byte[0] = Addr.Byte[0];                          // address of the register
  TempLong.Byte[1] = Addr.Byte[1];                          // to read, LsByte first
  TempLong.Byte[2] = Len;                                   // number of bytes to read
  TempLong.Byte[3] = ESC_READ;                              // ESC read 

  SPIWriteRegisterDirect (ECAT_CSR_CMD, TempLong.Long);     // write the command

  do
  {                                                         // wait for command execution
    TempLong.Long = SPIReadRegisterDirect(ECAT_CSR_CMD,4);  //
  }                                                         //
  while(TempLong.Byte[3] & ECAT_CSR_BUSY);                  //
                                                             
                                                              
  TempLong.Long = SPIReadRegisterDirect(ECAT_CSR_DATA,Len); // read the requested register
  return TempLong.Long;                                     //
}


//---- write an undirectly addressable registers  -------------------------------------------------

void  SPIWriteRegisterIndirect (unsigned long DataOut, unsigned short Address)

                                                   // Address = register to write
                                                   // DataOut = data to write                                                    
{
  ULONG TempLong;
  UWORD Addr;

  SPIWriteRegisterDirect (ECAT_CSR_DATA, DataOut);            // write the data

                                                              // compose the command
                                                              //                                
  TempLong.Byte[0] = Addr.Byte[0];                            // address of the register  
  TempLong.Byte[1] = Addr.Byte[1];                            // to write, LsByte first
  TempLong.Byte[2] = 4;                                       // we write always 4 bytes
  TempLong.Byte[3] = ESC_WRITE;                               // ESC write

  SPIWriteRegisterDirect (ECAT_CSR_CMD, TempLong.Long);       // write the command

  do                                                          // wait for command execution
  {                                                           //
    TempLong.Long = SPIReadRegisterDirect (ECAT_CSR_CMD, 4);  //  
  }                                                           //  
  while (TempLong.Byte[3] & ECAT_CSR_BUSY);                   //
  
}


//---- read from process ram fifo ----------------------------------------------------------------

void SPIReadProcRamFifo()             // read 32 bytes from the output process ram, through the fifo
                                      //        
                                      // these are the bytes received from the EtherCAT master and
                                      // that will be use by our application to write the outputs
{
  ULONG TempLong;
  unsigned char i;
  
  SPIWriteRegisterDirect (ECAT_PRAM_RD_ADDR_LEN, 0x00201000);   // we always read 32 bytes   0x0020----
                                                                // output process ram offset 0x----1000
                                                                
  SPIWriteRegisterDirect (ECAT_PRAM_RD_CMD, 0x80000000);        // start command                                                                  
                                                                                                                               
  do                                                            // wait for data to be transferred      
  {                                                             // from the output process ram 
    TempLong.Long = SPIReadRegisterDirect (ECAT_PRAM_RD_CMD,4); // to the read fifo 
  }                                                             //    
  while (!(TempLong.Byte[0] & PRAM_READ_AVAIL) || (TempLong.Byte[1] != 8));     
  
 
  Spi1Cs_ = 0;                                                  // SPI chip select enable  
  
  SPI1Transfer(COMM_SPI_READ);                                  // SPI read command
  SPI1Transfer(0x00);                                           // address of the read  
  SPI1Transfer(0x00);                                           // fifo MsByte first
  

  for (i=0; i<32; i++)                                          // 32 bytes read loop 
  {                                                             // 
    Out.Byte[i] = SPI1Transfer(DUMMY_BYTE);                     //
  }                                                             //
    
  Spi1Cs_ = 1;                                                  // SPI chip select disable    
}


//---- write to the process ram fifo --------------------------------------------------------------

void SPIWriteProcRamFifo()             // write 32 bytes to the input process ram, through the fifo
                                       //    
                                       // these are the bytes that we have read from the inputs of our                   
                                       // application and that will be sent to the EtherCAT master
{
  ULONG TempLong;
  unsigned char i;

  SPIWriteRegisterDirect (ECAT_PRAM_WR_ADDR_LEN, 0x00201200);   // we always write 32 bytes 0x0020----
                                                                // input process ram offset 0x----1200

  SPIWriteRegisterDirect (ECAT_PRAM_WR_CMD, 0x80000000);        // start command  

  do                                                            // check fifo has available space     
  {                                                             // for data to be written
    TempLong.Long = SPIReadRegisterDirect (ECAT_PRAM_WR_CMD,4); //  
  }                                                             //    
  while (!(TempLong.Byte[0] & PRAM_WRITE_AVAIL) || (TempLong.Byte[1] < 8) );             
  
  
  Spi1Cs_ = 0;                                                  // enable SPI chip select
  
  SPI1Transfer(COMM_SPI_WRITE);                                 // SPI write command
  SPI1Transfer(0x00);                                           // address of the write fifo 
  SPI1Transfer(0x20);                                           // MsByte first 

  for (i=0; i<32; i++)                                          // 32 bytes write loop
  {                                                             //
     SPI1Transfer (In.Byte[i]);                                 // 
  }                                                             //
                                                                //  

  Spi1Cs_ = 1;                                                  // disable SPI chip select      
} 



//- SPI 1 data transfer ------------------------------------------------------------------

unsigned char SPI1Transfer (unsigned char Value)

{
    unsigned char Result;

    SSP1BUF = (Value);
    while(!SSP1STATbits.BF){};
    Result = SSP1BUF;

    return Result;
}



